package sun.awt.image;

import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.image.RasterFormatException;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.ComponentSampleModel;
import java.awt.image.PixelInterleavedSampleModel;
import java.awt.image.DataBufferUShort;
import java.awt.image.DataBuffer;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.image.SampleModel;

public class ShortInterleavedRaster extends ShortComponentRaster
{
    private int maxX;
    private int maxY;
    
    public ShortInterleavedRaster(SampleModel sampleModel, Point point) {
        this(sampleModel, sampleModel.createDataBuffer(), new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public ShortInterleavedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        this(sampleModel, dataBuffer, new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public ShortInterleavedRaster(SampleModel obj, DataBuffer dataBuffer, Rectangle rectangle, Point point, ShortInterleavedRaster shortInterleavedRaster) {
        super(obj, dataBuffer, rectangle, point, shortInterleavedRaster);
        this.maxX = this.minX + this.width;
        this.maxY = this.minY + this.height;
        if (!(dataBuffer instanceof DataBufferUShort)) {
            throw new RasterFormatException("ShortInterleavedRasters must have ushort DataBuffers");
        }
        DataBufferUShort dataBufferUShort = (DataBufferUShort)dataBuffer;
        this.data = SunWritableRaster.stealData(dataBufferUShort, 0);
        if (obj instanceof PixelInterleavedSampleModel || (obj instanceof ComponentSampleModel && obj.getNumBands() == 1)) {
            ComponentSampleModel componentSampleModel = (ComponentSampleModel)obj;
            this.scanlineStride = componentSampleModel.getScanlineStride();
            this.pixelStride = componentSampleModel.getPixelStride();
            this.dataOffsets = componentSampleModel.getBandOffsets();
            int n4 = rectangle.x - point.x;
            int n5 = rectangle.y - point.y;
            for (int i = 0; i < this.getNumDataElements(); ++i) {
                int[] dataOffsets2 = this.dataOffsets;
                int n6 = i;
                dataOffsets2[n6] += n4 * this.pixelStride + n5 * this.scanlineStride;
            }
        }
        else if ((obj instanceof SinglePixelPackedSampleModel)) {
            this.scanlineStride = ((SinglePixelPackedSampleModel)obj).getScanlineStride();
            this.pixelStride = 1;
            (this.dataOffsets = new int[1])[0] = dataBufferUShort.getOffset();
            int n = rectangle.x - point.x;
            int n2 = rectangle.y - point.y;
            int[] dataOffsets = this.dataOffsets;
            int n3 = 0;
            dataOffsets[n3] += n + n2 * this.scanlineStride;
        }
        else {
            throw new RasterFormatException("ShortInterleavedRasters must have PixelInterleavedSampleModel, SinglePixelPackedSampleModel or 1 band ComponentSampleModel.  Sample model is " + obj);
        }
        this.bandOffset = this.dataOffsets[0];
        this.verify();
        return;
    }
    
    @Override
    public int[] getDataOffsets() {
        return (int[])(int[])this.dataOffsets.clone();
    }
    
    @Override
    public int getDataOffset(int n) {
        return this.dataOffsets[n];
    }
    
    @Override
    public int getScanlineStride() {
        return this.scanlineStride;
    }
    
    @Override
    public int getPixelStride() {
        return this.pixelStride;
    }
    
    @Override
    public short[] getDataStorage() {
        return this.data;
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        short[] array;
        if (o == null) {
            array = new short[this.numDataElements];
        }
        else {
            array = (short[])(short[])o;
        }
        int n3 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        for (int i = 0; i < this.numDataElements; ++i) {
            array[i] = this.data[this.dataOffsets[i] + n3];
        }
        return array;
    }
    
    @Override
    public Object getDataElements(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        short[] array;
        if (o == null) {
            array = new short[n3 * n4 * this.numDataElements];
        }
        else {
            array = (short[])(short[])o;
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        int n6 = 0;
        for (int i = 0; i < n4; ++i, n5 += this.scanlineStride) {
            for (int j = 0; j < n3; ++j, n5 += this.pixelStride) {
                for (int k = 0; k < this.numDataElements; ++k) {
                    array[n6++] = this.data[this.dataOffsets[k] + n5];
                }
            }
        }
        return array;
    }
    
    @Override
    public short[] getShortData(int n, int n2, int n3, int n4, int n5, short[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new short[this.numDataElements * n3 * n4];
        }
        int n6 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride + this.dataOffsets[n5];
        int n7 = 0;
        if (this.pixelStride == 1) {
            if (this.scanlineStride == n3) {
                System.arraycopy(this.data, n6, array, 0, n3 * n4);
            }
            else {
                for (int k = 0; k < n4; n7 += n3, ++k, n6 += this.scanlineStride) {
                    System.arraycopy(this.data, n6, array, n7, n3);
                }
            }
        }
        else {
            for (int i = 0; i < n4; ++i, n6 += this.scanlineStride) {
                for (int j = 0; j < n3; ++j, n6 += this.pixelStride) {
                    array[n7++] = this.data[n6];
                }
            }
        }
        return array;
    }
    
    @Override
    public short[] getShortData(int n, int n2, int n3, int n4, short[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new short[this.numDataElements * n3 * n4];
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        int n6 = 0;
        for (int i = 0; i < n4; ++i, n5 += this.scanlineStride) {
            for (int j = 0; j < n3; ++j, n5 += this.pixelStride) {
                for (int k = 0; k < this.numDataElements; ++k) {
                    array[n6++] = this.data[this.dataOffsets[k] + n5];
                }
            }
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        short[] array = (short[])(short[])o;
        int n3 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        for (int i = 0; i < this.numDataElements; ++i) {
            this.data[this.dataOffsets[i] + n3] = array[i];
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, Raster raster) {
        int n3 = n + raster.getMinX();
        int n4 = n2 + raster.getMinY();
        int width = raster.getWidth();
        int height = raster.getHeight();
        if (n3 < this.minX || n4 < this.minY || n3 + width > this.maxX || n4 + height > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        this.setDataElements(n3, n4, width, height, raster);
        return;
    }
    
    private void setDataElements(int n, int n2, int w, int n3, Raster raster) {
        if (w <= 0 || n3 <= 0) {
            return;
        }
        int minX = raster.getMinX();
        int minY = raster.getMinY();
        Object dataElements = null;
        for (int i = 0; i < n3; ++i) {
            dataElements = raster.getDataElements(minX, minY + i, w, 1, dataElements);
            this.setDataElements(n, n2 + i, w, 1, dataElements);
        }
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        short[] array = (short[])(short[])o;
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        int n6 = 0;
        for (int i = 0; i < n4; ++i, n5 += this.scanlineStride) {
            for (int j = 0; j < n3; ++j, n5 += this.pixelStride) {
                for (int k = 0; k < this.numDataElements; ++k) {
                    this.data[this.dataOffsets[k] + n5] = array[n6++];
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void putShortData(int n, int n2, int n3, int n4, int n5, short[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n6 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride + this.dataOffsets[n5];
        int n7 = 0;
        if (this.pixelStride == 1) {
            if (this.scanlineStride == n3) {
                System.arraycopy(array, 0, this.data, n6, n3 * n4);
            }
            else {
                for (int k = 0; k < n4; n7 += n3, ++k, n6 += this.scanlineStride) {
                    System.arraycopy(array, n7, this.data, n6, n3);
                }
            }
        }
        else {
            for (int i = 0; i < n4; ++i, n6 += this.scanlineStride) {
                for (int j = 0; j < n3; ++j, n6 += this.pixelStride) {
                    this.data[n6] = array[n7++];
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void putShortData(int n, int n2, int n3, int n4, short[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) * this.pixelStride;
        int n6 = 0;
        for (int i = 0; i < n4; ++i, n5 += this.scanlineStride) {
            for (int j = 0; j < n3; ++j, n5 += this.pixelStride) {
                for (int k = 0; k < this.numDataElements; ++k) {
                    this.data[this.dataOffsets[k] + n5] = array[n6++];
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public Raster createChild(int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        return this.createWritableChild(n, n2, n3, n4, n5, n6, array);
    }
    
    @Override
    public WritableRaster createWritableChild(int n, int n2, int width, int height, int x, int y, int[] array) {
        if (n < this.minX) {
            throw new RasterFormatException("x lies outside the raster");
        }
        if (n2 < this.minY) {
            throw new RasterFormatException("y lies outside the raster");
        }
        if (n + width < n || n + width > this.minX + this.width) {
            throw new RasterFormatException("(x + width) is outside of Raster");
        }
        if (n2 + height < n2 || n2 + height > this.minY + this.height) {
            throw new RasterFormatException("(y + height) is outside of Raster");
        }
        SampleModel sampleModel;
        if (array != null) {
            sampleModel = this.sampleModel.createSubsetSampleModel(array);
        }
        else {
            sampleModel = this.sampleModel;
        }
        return new ShortInterleavedRaster(sampleModel, this.dataBuffer, new Rectangle(x, y, width, height), new Point(this.sampleModelTranslateX + (x - n), this.sampleModelTranslateY + (y - n2)), this);
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        if (n <= 0 || n2 <= 0) {
            throw new RasterFormatException("negative " + ((n <= 0) ? "width" : "height"));
        }
        return new ShortInterleavedRaster(this.sampleModel.createCompatibleSampleModel(n, n2), new Point(0, 0));
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster() {
        return this.createCompatibleWritableRaster(this.width, this.height);
    }
    
    @Override
    public String toString() {
        return new String("ShortInterleavedRaster: width = " + this.width + " height = " + this.height + " #numDataElements " + this.numDataElements);
    }
}
